@@ -11,8 +11,6 @@ import time
 
 import numpy as np
 
-from sklearn.base import BaseEstimator
-from sklearn.exceptions import NotFittedError
 from sklearn.utils import check_array
 
 from ..logging import Logger
@@ -20,6 +18,7 @@ from ..preprocessing import preprocessing_user_splits_categorical
 from ..preprocessing import split_data
 from .auto_monotonic import auto_monotonic
 from .auto_monotonic import peak_valley_trend_change_heuristic
+from .base import BaseOptimalBinning
 from .binning_information import print_binning_information
 from .binning_statistics import bin_categorical
 from .binning_statistics import bin_info
@@ -38,8 +37,9 @@ def _check_parameters(name, dtype, prebinning_method, solver, max_n_prebins,
                       min_bin_n_event, max_bin_n_event, monotonic_trend,
                       min_event_rate_diff, max_pvalue, max_pvalue_policy,
                       gamma, outlier_detector, outlier_params, class_weight,
-                      cat_cutoff, user_splits, special_codes, split_digits,
-                      mip_solver, time_limit, verbose):
+                      cat_cutoff, user_splits, user_splits_fixed,
+                      special_codes, split_digits, mip_solver, time_limit,
+                      verbose):
 
     if not isinstance(name, str):
         raise TypeError("name must be a string.")
@@ -141,7 +141,7 @@ def _check_parameters(name, dtype, prebinning_method, solver, max_n_prebins,
             raise ValueError('Invalid value for monotonic trend. Allowed '
                              'string values are "auto", "auto_heuristic", '
                              '"auto_asc_desc", "ascending", "descending", '
-                             '"concave", "convex", "peak" "valley", '
+                             '"concave", "convex", "peak", "valley", '
                              '"peak_heuristic" and "valley_heuristic".')
 
     if (not isinstance(min_event_rate_diff, numbers.Number) or
@@ -191,6 +191,21 @@ def _check_parameters(name, dtype, prebinning_method, solver, max_n_prebins,
         if not isinstance(user_splits, (np.ndarray, list)):
             raise TypeError("user_splits must be a list or numpy.ndarray.")
 
+    if user_splits_fixed is not None:
+        if user_splits is None:
+            raise ValueError("user_splits must be provided.")
+        else:
+            if not isinstance(user_splits_fixed, (np.ndarray, list)):
+                raise TypeError("user_splits_fixed must be a list or "
+                                "numpy.ndarray.")
+            elif not all(isinstance(s, bool) for s in user_splits_fixed):
+                raise ValueError("user_splits_fixed must be list of boolean.")
+            elif len(user_splits) != len(user_splits_fixed):
+                raise ValueError("Inconsistent length of user_splits and "
+                                 "user_splits_fixed: {} != {}. Lengths must "
+                                 "be equal".format(len(user_splits),
+                                                   len(user_splits_fixed)))
+
     if special_codes is not None:
         if not isinstance(special_codes, (np.ndarray, list)):
             raise TypeError("special_codes must be a list or numpy.ndarray.")
@@ -213,7 +228,7 @@ def _check_parameters(name, dtype, prebinning_method, solver, max_n_prebins,
         raise TypeError("verbose must be a boolean; got {}.".format(verbose))
 
 
-class OptimalBinning(BaseEstimator):
+class OptimalBinning(BaseOptimalBinning):
     """Optimal binning of a numerical or categorical variable with respect to a
     binary target.
 
@@ -338,6 +353,11 @@ class OptimalBinning(BaseEstimator):
         The list of pre-binning split points when ``dtype`` is "numerical" or
         the list of prebins when ``dtype`` is "categorical".
 
+    user_splits_fixed : array-like or None (default=None)
+        The list of pre-binning split points that must be fixed.
+
+        .. versionadded:: 0.5.0
+
     special_codes : array-like or None, optional (default=None)
         List of special codes. Use special codes to specify the data values
         that must be treated separately.
@@ -382,9 +402,9 @@ class OptimalBinning(BaseEstimator):
                  min_event_rate_diff=0, max_pvalue=None,
                  max_pvalue_policy="consecutive", gamma=0,
                  outlier_detector=None, outlier_params=None, class_weight=None,
-                 cat_cutoff=None, user_splits=None, special_codes=None,
-                 split_digits=None, mip_solver="bop", time_limit=100,
-                 verbose=False):
+                 cat_cutoff=None, user_splits=None, user_splits_fixed=None,
+                 special_codes=None, split_digits=None, mip_solver="bop",
+                 time_limit=100, verbose=False):
 
         self.name = name
         self.dtype = dtype
@@ -416,6 +436,7 @@ class OptimalBinning(BaseEstimator):
         self.cat_cutoff = cat_cutoff
 
         self.user_splits = user_splits
+        self.user_splits_fixed = user_splits_fixed
         self.special_codes = special_codes
         self.split_digits = split_digits
 
@@ -436,6 +457,8 @@ class OptimalBinning(BaseEstimator):
         self._n_nonevent_cat_others = None
         self._n_event_cat_others = None
         self._problem_type = "classification"
+        self._user_splits = user_splits
+        self._user_splits_fixed = user_splits_fixed
 
         # info
         self._binning_table = None
@@ -858,7 +881,7 @@ class OptimalBinning(BaseEstimator):
                                   self.max_bin_n_nonevent,
                                   self.min_event_rate_diff, self.max_pvalue,
                                   self.max_pvalue_policy, self.gamma,
-                                  self.time_limit)
+                                  self._user_splits_fixed, self.time_limit)
         elif self.solver == "mip":
             optimizer = BinningMIP(monotonic, self.min_n_bins, self.max_n_bins,
                                    min_bin_size, max_bin_size,
@@ -867,7 +890,8 @@ class OptimalBinning(BaseEstimator):
                                    self.max_bin_n_nonevent,
                                    self.min_event_rate_diff, self.max_pvalue,
                                    self.max_pvalue_policy, self.gamma,
-                                   self.mip_solver, self.time_limit)
+                                   self._user_splits_fixed, self.mip_solver,
+                                   self.time_limit)
         elif self.solver == "ls":
             optimizer = BinningLS(monotonic, self.min_n_bins, self.max_n_bins,
                                   min_bin_size, max_bin_size,
@@ -875,7 +899,8 @@ class OptimalBinning(BaseEstimator):
                                   self.min_bin_n_nonevent,
                                   self.max_bin_n_nonevent,
                                   self.min_event_rate_diff, self.max_pvalue,
-                                  self.max_pvalue_policy, self.time_limit)
+                                  self.max_pvalue_policy,
+                                  self._user_splits_fixed, self.time_limit)
 
         if self.verbose:
             logging.info("Optimizer: build model...")
@@ -891,7 +916,11 @@ class OptimalBinning(BaseEstimator):
 
         self._optimizer = optimizer
         self._status = status
-        self._splits_optimal = splits[solution[:-1]]
+
+        if self.dtype == "categorical" and self.user_splits is not None:
+            self._splits_optimal = splits[solution]
+        else:
+            self._splits_optimal = splits[solution[:-1]]
 
         self._time_solver = time.perf_counter() - time_init
 
@@ -931,23 +960,20 @@ class OptimalBinning(BaseEstimator):
 
         return splits_prebinning, n_nonevent, n_event
 
-    def _check_is_fitted(self):
-        if not self._is_fitted:
-            raise NotFittedError("This {} instance is not fitted yet. Call "
-                                 "'fit' with appropriate arguments."
-                                 .format(self.__class__.__name__))
-
     def _compute_prebins(self, splits_prebinning, x, y0, y1):
         n_splits = len(splits_prebinning)
-
         if not n_splits:
             return splits_prebinning, np.array([]), np.array([])
 
-        indices = np.digitize(x, splits_prebinning, right=False)
+        if self.dtype == "categorical" and self.user_splits is not None:
+            indices = np.digitize(x, splits_prebinning, right=True)
+            n_bins = n_splits
+        else:
+            indices = np.digitize(x, splits_prebinning, right=False)
+            n_bins = n_splits + 1
 
-        n_bins = n_splits + 1
-        n_nonevent = np.zeros(n_bins).astype(np.int)
-        n_event = np.zeros(n_bins).astype(np.int)
+        n_nonevent = np.empty(n_bins).astype(np.int64)
+        n_event = np.empty(n_bins).astype(np.int64)
 
         for i in range(n_bins):
             mask = (indices == i)
@@ -959,8 +985,26 @@ class OptimalBinning(BaseEstimator):
         if np.any(mask_remove):
             self._n_refinements += 1
 
-            mask_splits = np.concatenate(
-                [mask_remove[:-2], [mask_remove[-2] | mask_remove[-1]]])
+            if self.dtype == "categorical" and self.user_splits is not None:
+                mask_splits = mask_remove
+            else:
+                mask_splits = np.concatenate(
+                    [mask_remove[:-2], [mask_remove[-2] | mask_remove[-1]]])
+
+            if self.user_splits_fixed is not None:
+                user_splits_fixed = np.asarray(self._user_splits_fixed)
+                user_splits = np.asarray(self._user_splits)
+                fixed_remove = user_splits_fixed & mask_splits
+
+                if any(fixed_remove):
+                    raise ValueError("Fixed user_splits {} are removed "
+                                     "because produce pure prebins. Provide "
+                                     "different splits to be fixed."
+                                     .format(user_splits[fixed_remove]))
+
+                # Update boolean array of fixed user splits.
+                self._user_splits_fixed = user_splits_fixed[~mask_splits]
+                self._user_splits = user_splits[~mask_splits]
 
             splits = splits_prebinning[~mask_splits]
 
